require 'lib/configfile/samba'
require 'main/container'

Figlet = :SambaFiglet

    include ConfigFile::Samba

    UnixPath = '/drives/Volume1/'
## This will give you a full list of character sets supported by Samba for 
## 'unix charset' and 'display charset'
##
## But we actually define only subset of them below
##
#    UnixCharsets = open("| iconv -l") do |f|
#	f.readlines[5..-1].collect { |x| x.sub(/\/\/\n/,"") }
#    end
#
    UnixCharsets = {'ISO_8859-1'=>'Latin-1', "ISO_8859-2"=>'Latin-2', 
    	"ISO_8859-3"=>'Latin-3', "ISO_8859-4"=>'Latin-4', 
	 "ISO_8859-7"=>'Greek', "ISO_8859-8"=>'Hebrew', 
	 "ISO_8859-9"=>'Latin-5', "ISO_8859-15"=>'Latin-1@Euro',
	 "ISO_8859-5"=>'Cyrillic (ISO)', "KOI8-R"=>'Cyrillic (KOI8-R)',
	 "KOI8-U"=>'Cyrillic (KOI8-U)', "CP1251"=>'Cyrillic (CP1251)', 
	 "PT154"=>'Cyrillic (PT154)', "SHIFT-JIS"=>'Japanese (Shift-JIS)',
	 "GB2312"=>'Simplified Chinese', "UTF-8"=>'UTF-8'}

#    UnixCharsets = ["ISO_8859-1", "ISO_8859-2", "ISO_8859-3", "ISO_8859-4", 
#	 "ISO_8859-7", "ISO_8859-8", "ISO_8859-9", "ISO_8859-15", "ISO_8859-5",
#	"KOI8-R", "KOI8-U", "CP1251", "PT154", "SHIFT-JIS", "GB2312", "UTF-8"
#    ]
#    UnixCharsetsDesc = ["Latin-1", "Latin-2", "Latin-3", "Latin-4", 
#	"Greek", "Hebrew", "Latin-5", "Latin-1@Euro", "Cyrillic (ISO)", 
#	"Cyrillic (KOI8-R)", "Cyrillic (KOI8-U)", "Cyrillic (CP1251)", 
#	"Cyrillic (PT154)", "Japanese - Shift-JIS", "Simplified Chinese", "UTF-8"
#    ]
    DosCharsets = {"CP437"=>'English (US)', "CP737"=>'Greek', 
    	"CP850"=>'Western European', "CP852"=>'Eastern European', 
	"CP862"=>'Hebrew', "CP861"=>'Icelandic', "CP866"=>'Cyrillic (Russian)',
	"CP1125"=>'Cyrillic (Ukrainian)', "CP932"=>'Japanese (Shift-JIS)',
	"CP936"=>'Simplified Chinese', "CP949"=>'Korean Hangul',
	"CP950"=>'Traditional Chinese'}
    
#    DosCharsets = ["CP437", "CP737", "CP850", "CP852", "CP862", "CP861", "CP866", "CP1125", "CP932", "CP936", "CP949", "CP950", ""]
#    DosCharsetsDesc = ["English (US)", "Greek", "Western European", "Eastern European", "Hebrew",
#	"Icelandic", "Cyrillic - Russian", "Cyrillic - Ukrainian", "Japanese - Shift-JIS", 
#	"Simplified Chinese", "Korean Hangul", "Traditional Chinese"
#    ]
    
    class SambaFiglet < Core::Figlet
	attr_reader :cnf
	
	def initialize
	    super("Samba")
	    @filenames = ["smb.conf"]
	    @base_path = "/etc/samba/"
	    @cnf = nil
	    loadconfig
	    main
	end
	
	def save
	    saveconfig
	    reloadservice
	end

	def load
	    loadconfig
	    reloadservice
	    main
	end
	
	def loadconfig
	    if File.exist?(@base_path + @filenames[0])
		@cnf = ConfigFile::Samba::SambaConfig.new(@base_path + @filenames[0])
	    else
		@cnf = ConfigFile::Samba::SambaConfig.new(@base_path + @filenames[0])
	    end
	end
	
	def saveconfig
	    File.open(@base_path + @filenames[0], "w") do |file|
		file.print @cnf
	    end
	end
	
	def reloadservice
	    Kernel::service_run(:smb,:reload)
	end

	def startservice
	    Kernel::service_run(:smb,:start)
	end

	def stopservice
	    Kernel::service_run(:smb,:stop)
	end
	
	def main
	    @hierarchy = nil
	    ## constructing the hierarchy
	    @hierarchy = Core::Notebook.new("Samba")
	    @hierarchy.residence = self
	    
	    GlobalShare.new('samba_global', @hierarchy, @cnf["global"])
	    list = SharesList.new("samba_top", @hierarchy, @cnf)
	    @cnf.each_value do |v|
		if v.to_key != 'global'
		    Share.new(v.to_key, list, v)
		end
	    end

	    Core::Action.new("Write config", @hierarchy) do |x| 
		save
		@hierarchy
	    end

	    # start and stop service buttons
    	    @status_msg = Core::Text.new("Service status",@hierarchy,"")
    	    if get_status
		@status_str = 'Running...'
	    else
		@status_str = 'Stopped...'
	    end
    	    @stop_button = Core::Action.new("Service stop", @hierarchy) { |x| 
		stopservice
		unless get_status
        	    @status_msg.set 'Stopped...'
		end	
		@hierarchy
	    }
    	    @start_button = Core::Action.new("Service start", @hierarchy) { |x| 
	        startservice
	        if get_status
    		    @status_msg.set 'Running...'
		end	
		@hierarchy
	    }
    	    @status_msg.set @status_str
	end
	
	def get_status
    	    fname = '/var/lock/subsys/smb'
    	    File.exist?(fname)
        end	
    end

    class Share < Core::Notebook
	def initialize (name, parent, options, newshare=false)
	    raise "Parent must be SharesList" unless SharesList===parent or parent.nil?
	    super(name, parent)
	    @newshare = newshare
	    @options = options
	    Core::Leaf.new("smb_share name", self).set options.to_key
	    ["comment", "valid users"].each do |opt|
		option = options[opt]
		Core::Leaf.new("smb_cfg_#{opt}", self).set option.option if option
	    end
	    ["public", "writable", "printable", "browseable"].each do |opt|
		option = options[opt]
		Core::Boolean.new("smb_cfg_#{opt}", self, option.option.downcase) if option
	    end
	    Core::Action.new("Update", self) do |x| 
		oldpath = options.section_name
		x.each_pair {|k, v|
		    case k.name
			when /^smb_cfg_/
		    	    options[k.name.sub(/smb_cfg_/,"")].option = k.from_s(v)
			    k.set(k.from_s(v))
			when /^smb_share name/
			    options.section_name = k.from_s(v)
			    if options.section_name.empty?
				options.section_name = "__empty__"
			    end
			    k.set(options.section_name)
		    end
		}
		if oldpath != options.section_name
		    begin
			File.stat(UnixPath+oldpath)
			File.rename(UnixPath+oldpath, UnixPath+options.section_name)
			rescue
			begin
			    Dir.mkdir(UnixPath+options.section_name, 0777)
			rescue
			end
		    end
		end
		if options.section_name !~ /^homes$/
		    options["path"].option = UnixPath+options.section_name
		end
		self.label  = options.section_name
		@name = options.section_name
	    	if @residence.cnf.add_option(options) 
                        @newshare=false if @newshare
                end
		@parent
	    end
	end
	
	def actionDelete
	    @residence.cnf.del_option(@options.to_key)
	    oldparent = @parent
	    oldparent.delChild(self)
	    oldparent
	end

	# Delete share if user moves to another page without update
	def postDisplay
	  parent.delChild(self) if parent and @newshare
	end

    end

    class GlobalShare < Core::Notebook
	def initialize (name, parent, options)
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent)
	    
	    [ "workgroup", "password server" ].each do |opt|
		option = options[opt]
		Core::Leaf.new("smb_cfg_#{opt}", self).set option.option if option
	    end
	    
	    # Don't show 'display charset' in UI as this might confuse user
	    # and both 'unix charset' and 'display charset' must be the same
	    # in our case
	    [ "unix charset" ].each do |opt|
		option = options[opt]
		Core::Variant.new("smb_cfg_#{opt}", UnixCharsets.values, self).
		    set UnixCharsets[option.option.upcase] if option
	    end
	    
	    doscharset=options["dos charset"]
	    Core::Variant.new("smb_cfg_dos charset", DosCharsets.values, self).
		set DosCharsets[doscharset.option.upcase] if doscharset
		
	    [ "domain master", "local master", "wins support" ].each do |opt|
		option = options[opt]
		Core::Boolean.new("smb_cfg_#{opt}", self, option.option.downcase) if option
	    end
	    
	    security=options["security"]
	    Core::Variant.new("smb_cfg_security", ["share","user", "domain", "ads"], self, security.option.downcase) if security
	    Core::Action.new("Update", self) do |x| 
		# Don't forget to add conversion back from descriptions to values
		# for 'unix charser' / 'dos charset' / 'display charset'
		x.each_pair {|k, v| 
		    case k.name
			when "smb_cfg_unix charset"
			    val = UnixCharsets.invert[k.from_s(v)]
			    options["display charset"].option = val
			    options["unix charset"].option = val
			when "smb_cfg_dos charset"
			    val = DosCharsets.invert[k.from_s(v)]
			    options["dos charset"].option = val
			when /^smb_cfg_/
			    options[k.name.sub(/smb_cfg_/,"")].option = k.from_s(v)
		    end
		    k.set(k.from_s(v))
		}
		@parent
	    end
	end
    end

    class SharesList < Core::MutableList
	
	def initialize(name, parent, conf)
	    super(name, parent) {|x| Share === x}
	    @conf = conf #store a configurator
	end

	def actionAdd
	    ss = ConfigFile::Samba::SambaSection.new ([], "share_#{@conf.keys.length-1}")
	    ss.add_option ConfigFile::Samba::SambaOption.new("path", "#{UnixPath}#{ss.section_name}")
	    ss.add_option ConfigFile::Samba::SambaOption.new("public", 'no')
	    ss.add_option ConfigFile::Samba::SambaOption.new("writable", 'no')
	    ss.add_option ConfigFile::Samba::SambaOption.new("printable", 'no')
	    ss.add_option ConfigFile::Samba::SambaOption.new("browseable", 'no')
	    ss.add_option ConfigFile::Samba::SambaOption.new("valid users", '@users')
	    ss.add_option ConfigFile::Samba::SambaOption.new("comment", '')
	    share = Share.new(ss.section_name, self, ss, true)
	    begin
		Dir.mkdir(UnixPath+ss.section_name)
	    rescue
	    end
	    share
	end

    end

